@@ -3,24 +3,24 @@ This requires Xonsh installed in venv or otherwise available on PATH
 """
 import os
 import shutil
-import tempfile
 import subprocess as sp
+import tempfile
+from pathlib import Path
 
 import pytest
 
 import xonsh
-from xonsh.lib.os import indir
-
-from tools import (
-    skip_if_on_msys,
-    skip_if_on_windows,
-    skip_if_on_darwin,
-    ON_WINDOWS,
+from xonsh.dirstack import with_pushd
+from xonsh.pytest.tools import (
     ON_DARWIN,
     ON_TRAVIS,
+    ON_WINDOWS,
+    skip_if_on_darwin,
+    skip_if_on_msys,
+    skip_if_on_unix,
+    skip_if_on_windows,
 )
 
-
 PATH = (
     os.path.join(os.path.abspath(os.path.dirname(__file__)), "bin")
     + os.pathsep
@@ -46,12 +46,17 @@ def run_xonsh(
     stderr=sp.STDOUT,
     single_command=False,
     interactive=False,
+    path=None,
 ):
     env = dict(os.environ)
-    env["PATH"] = PATH
+    if path is None:
+        env["PATH"] = PATH
+    else:
+        env["PATH"] = path
     env["XONSH_DEBUG"] = "0"  # was "1"
     env["XONSH_SHOW_TRACEBACK"] = "1"
     env["RAISE_SUBPROC_ERROR"] = "0"
+    env["FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE"] = "1"
     env["PROMPT"] = ""
     xonsh = shutil.which("xonsh", path=PATH)
     args = [xonsh, "--no-rc"]
@@ -512,6 +517,77 @@ first
         lambda out: 'Recursive calls to "first" alias.' in out,
         0,
     ),
+    # testing alias stack: parallel threaded callable aliases.
+    # This breaks if the __ALIAS_STACK variables leak between threads.
+    (
+        """
+from time import sleep
+aliases['a'] = lambda: print(1, end="") or sleep(0.2) or print(1, end="")
+aliases['b'] = 'a'
+a | a
+a | a
+a | b | a
+a | a | b | b
+""",
+        "1" * 2 * 4,
+        0,
+    ),
+    # test $SHLVL
+    (
+        """
+# test parsing of $SHLVL
+
+$SHLVL = "1"
+echo $SHLVL # == 1
+
+$SHLVL = 1
+echo $SHLVL # == 1
+
+$SHLVL = "-13"
+echo $SHLVL # == 0
+
+$SHLVL = "error"
+echo $SHLVL # == 0
+
+$SHLVL = 999
+echo $SHLVL # == 999
+
+$SHLVL = 1000
+echo $SHLVL # == 1
+
+# sourcing a script should maintain $SHLVL
+
+$SHLVL = 5
+touch temp_shlvl_test.sh
+source-bash temp_shlvl_test.sh
+rm temp_shlvl_test.sh
+echo $SHLVL # == 5
+
+# creating a subshell should increment the child's $SHLVL and maintain the parents $SHLVL
+
+$SHLVL = 5
+xonsh -c r'echo $SHLVL' # == 6
+echo $SHLVL # == 5
+
+# replacing the current process with another process should derease $SHLVL
+# (so that if the new process is a shell, $SHLVL is maintained)
+
+$SHLVL = 5
+xexec python3 -c 'import os; print(os.environ["SHLVL"])' # == 4
+""",
+        """1
+1
+0
+0
+999
+1
+5
+6
+5
+4
+""",
+        0,
+    ),
 ]
 
 if not ON_WINDOWS:
@@ -520,6 +596,7 @@ if not ON_WINDOWS:
 
 @skip_if_no_xonsh
 @pytest.mark.parametrize("case", ALL_PLATFORMS)
+@pytest.mark.flaky(reruns=5, reruns_delay=2)
 def test_script(case):
     script, exp_out, exp_rtn = case
     out, err, rtn = run_xonsh(script)
@@ -579,8 +656,8 @@ def test_eof_syntax_error():
     """Ensures syntax errors for EOF appear on last line."""
     script = "x = 1\na = (1, 0\n"
     out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
-    assert ":0:0: EOF in multi-line statement" not in err
-    assert ":2:0: EOF in multi-line statement" in err
+    assert "line 0" not in err
+    assert "EOF in multi-line statement" in err and "line 2" in err
 
 
 @skip_if_no_xonsh
@@ -593,8 +670,8 @@ def test_open_quote_syntax_error():
         'echo "This is line 6"\n'
     )
     out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
-    assert """:3:5: ('code: "This is line 3"',)""" not in err
-    assert ':5:4: "' in err
+    assert """('code: "This is line 3"',)""" not in err
+    assert "line 5" in err
     assert "SyntaxError:" in err
 
 
@@ -679,7 +756,7 @@ def test_subshells(cmd, fmt, exp):
 @pytest.mark.parametrize("cmd, exp", [("pwd", lambda: os.getcwd() + "\n")])
 def test_redirect_out_to_file(cmd, exp, tmpdir):
     outfile = tmpdir.mkdir("xonsh_test_dir").join("xonsh_test_file")
-    command = "{} > {}\n".format(cmd, outfile)
+    command = f"{cmd} > {outfile}\n"
     out, _, _ = run_xonsh(command)
     content = outfile.read()
     if callable(exp):
@@ -705,7 +782,7 @@ def test_xonsh_no_close_fds():
         "b:\n"
         "\tsleep 1\n"
     )
-    with tempfile.TemporaryDirectory() as d, indir(d):
+    with tempfile.TemporaryDirectory() as d, with_pushd(d):
         with open("Makefile", "w") as f:
             f.write(makefile)
         out = sp.check_output(["make", "-sj2", "SHELL=xonsh"], universal_newlines=True)
@@ -720,7 +797,7 @@ def test_xonsh_no_close_fds():
     ],  # noqa E231 (black removes space)
 )
 def test_pipe_between_subprocs(cmd, fmt, exp):
-    "verify pipe between subprocesses doesn't throw an exception"
+    """verify pipe between subprocesses doesn't throw an exception"""
     check_run_xonsh(cmd, fmt, exp)
 
 
@@ -813,3 +890,36 @@ def test_loading_correctly(monkeypatch, interactive):
 def test_exec_function_scope(cmd):
     _, _, rtn = run_xonsh(cmd, single_command=True)
     assert rtn == 0
+
+
+@skip_if_on_unix
+def test_run_currentfolder(monkeypatch):
+    """Ensure we can run an executable in the current folder
+    when file is not on path
+    """
+    batfile = Path(__file__).parent / "bin" / "hello_world.bat"
+    monkeypatch.chdir(batfile.parent)
+    cmd = batfile.name
+    out, _, _ = run_xonsh(cmd, stdout=sp.PIPE, stderr=sp.PIPE, path=os.environ["PATH"])
+    assert out.strip() == "hello world"
+
+
+@skip_if_on_unix
+def test_run_dynamic_on_path():
+    """Ensure we can run an executable which is added to the path
+    after xonsh is loaded
+    """
+    batfile = Path(__file__).parent / "bin" / "hello_world.bat"
+    cmd = f"$PATH.add(r'{batfile.parent}');![hello_world.bat]"
+    out, _, _ = run_xonsh(cmd, path=os.environ["PATH"])
+    assert out.strip() == "hello world"
+
+
+@skip_if_on_unix
+def test_run_fail_not_on_path():
+    """Test that xonsh fails to run an executable when not on path
+    or in current folder
+    """
+    cmd = "hello_world.bat"
+    out, _, _ = run_xonsh(cmd, stdout=sp.PIPE, stderr=sp.PIPE, path=os.environ["PATH"])
+    assert out != "Hello world"
